@@ -17,9 +17,7 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -94,11 +92,8 @@ static void emit_icon_changed (WnckApplication *app);
 static void reset_name  (WnckApplication *app);
 static void update_name (WnckApplication *app);
 
-static void wnck_application_init        (WnckApplication      *application);
-static void wnck_application_class_init  (WnckApplicationClass *klass);
 static void wnck_application_finalize    (GObject        *object);
 
-
 static guint signals[LAST_SIGNAL] = { 0 };
 
 void
