@@ -18,9 +18,7 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -82,11 +80,8 @@ enum {
   LAST_SIGNAL
 };
 
-static void wnck_workspace_init        (WnckWorkspace      *workspace);
-static void wnck_workspace_class_init  (WnckWorkspaceClass *klass);
 static void wnck_workspace_finalize    (GObject        *object);
 
-
 static void emit_name_changed (WnckWorkspace *space);
 
 static guint signals[LAST_SIGNAL] = { 0 };
@@ -597,6 +592,9 @@ wnck_workspace_get_neighbor (WnckWorkspace       *space,
       else
         index += add;
       break;
+
+    default:
+      break;
     }
 
   if (index == space->priv->number)
